module Bindings (display,keyboard) where
 
import Graphics.UI.GLUT
import Data.IORef
import Display -- display functions

-- given the current parameters (pos, z and max) and a certain input do:
-- input == 'h' -> increase number of maximum iterations
keyboardAct pos z max (Char 'h') Down = do
  max' <- get max
  max $= max' + 3
-- input == 'l' -> decrease number of maximum iterations
keyboardAct pos z max (Char 'l') Down = do
  max' <- get max
  max $= max' - 3
-- input == '+' -> increase zoom factor
keyboardAct pos z max (Char '+') Down = do
  zoom' <- get z
  z $= zoom' + 0.1
-- input == '-' -> decrease zoom factor
keyboardAct pos z max (Char '-') Down = do
  zoom' <- get z
  z $= zoom' - 0.1
-- input == left arrow -> move current coordinates to left
keyboardAct p z max (SpecialKey KeyLeft) Down = do
  (x,y) <- get p
  p $= (x+0.2,y)
-- input == right arrow -> move current coordinates to right
keyboardAct p z max (SpecialKey KeyRight) Down = do
  (x,y) <- get p
  p $= (x-0.2,y)
-- input == down arrow -> move current coordinates to down
keyboardAct p z max (SpecialKey KeyUp) Down = do
  (x,y) <- get p
  p $= (x,y+0.2)
-- input == up arrow -> move current coordinates to up
keyboardAct p z max (SpecialKey KeyDown) Down = do
  (x,y) <- get p
  p $= (x,y-0.2)
-- otherwise ignore the input
keyboardAct _ _ _ _ _ = return ()

-- on key press ...
keyboard pos zoom max key state modifiers position = do
  keyboardAct pos zoom max key state -- ... amend the appropriate parameters ...
  postRedisplay Nothing -- ... and redisplay the whole function with the new values

