module Display (display) where

import Graphics.UI.GLUT
import Control.Monad
import Data.IORef
import Data.Complex
import Julia

x // y = fromIntegral x / fromIntegral y
-- given a function and some additional parameters produces the appropriate graphical representation
display :: (Complex Float -> Complex Float) -> IORef Int -> IORef (Float,Float) -> IORef Float -> IO() 
display function maxIt position zoom = 
	do
		clear [ ColorBuffer ];
		displayCallback $= (vertices >>= (displaycols function maxIt position zoom)); -- vertices produces a list of all vertices on the screen that is divided into sublists each containing one column of vertexes. The list of columns is given to displaycol which renders each column seperately. Thus, the screen is refreshed gradually from left to right.
		get currentWindow >>= postRedisplay

-- list of all vertexes in the window
vertices :: IO [[Vertex2 GLfloat]]
vertices = get windowSize >>= \(Size w h) ->				-- determine the size of the current window
           return $ [[pairToVertex (Size w h) (x, y) | y <- [0..h-1]] | x <- [0..w-1]] -- convert every pair of coordinates into a vertex and return the result

-- converts a pair of coordinates on a screen of given size to a vertex
pairToVertex :: Size -> (GLsizei, GLsizei) -> Vertex2 GLfloat
pairToVertex (Size w h) (x, y) = Vertex2 ((2 // w * fromIntegral x) - 1.0)		-- project the coordinates onto [-1,1] by putting there coordinates in relation to the maximal height and width
                             ((2 // h * fromIntegral y) - 1.0)

-- displays a list of vertices 
displaycols :: (Complex Float -> Complex Float) -> IORef Int -> IORef (Float,Float) -> IORef Float -> [[Vertex2 GLfloat]]-> IO() 
displaycols function maxIt position zoom cols  =
	do mapM_ (\col -> do {  renderPrimitive Points $ mapM_ (drawVertex function maxIt position zoom) col; flush }) cols; -- maps the render function of a single vertex on the given list of vertices

-- computes the output for one vertex
drawVertex :: (Complex Float -> Complex Float) -> IORef Int -> IORef (Float,Float) -> IORef Float -> Vertex2 GLfloat -> IO ()
drawVertex function maxIt position zoom v  = do {getColor function maxIt position zoom v; vertex v}

-- resolves all IO references and feeds them into the function determining the numbers of iterations needed to leave a circle with radius two in the plane of complex numbers. The number of steps is converted into an appropriate colour
getColor :: (Complex Float -> Complex Float) -> IORef Int -> IORef (Float,Float) -> IORef Float -> Vertex2 GLfloat -> IO ()
getColor func max position zoom v = do {z <- get zoom; (x,y) <- get position;maxIt <- get max; color $ colour (julia func maxIt (x,y) z v) maxIt }

