module Julia (colour, julia) where

import Graphics.UI.GLUT
import Control.Monad
import Data.Complex
import Data.Int


x // y = fromIntegral x / fromIntegral y

-- Determins number of iterations needed to leave a circle with radius 2. If the function values leave that circle, the sequence is bound to go towards infinity.
-- Otherwise we assume that the point is part of the Julia set.
julia :: (Complex Float -> Complex Float) -> Int -> (Float, Float)->  Float-> Vertex2 GLfloat -> Int
julia f maxIterations (x,y) zoom (Vertex2 r i)  = length . takeWhile (\z -> magnitude z <= 2) . take maxIterations $ iterate f ((zoom*(r+x)) :+ (zoom*(i+y)))
-- iterate the function infinitely often, only take a predefined number of steps, of these take only these where the value is greater or equal to 2. The number of steps taken is the number iterations we look for.



color3 r g b = Color3 r g b
-- given a number of iterations produce the appropriate colour
colour:: Int -> Int -> Color3 Float
colour num max | num == max = color3 0 0 0 -- iterations has not exceeded 2 when the maximum number of iterations was reached -> zhe specific point is assumed to be part of the Julia set and is painted black 
               | otherwise  = color3 (ratio*0.5) ratio (ratio*0.5) -- otherwise produce a green colour shade ... 
                 where ratio = num // max						-- that is lighter the more steps it takes to leave the circle of radius 2 (= the closer you are to the Julia set)


{- uncomment to activate alterntive Cosinus based colouring (and comment the function colour above)
d = 2/3*pi 

colour:: Int -> Int -> Color3 Float
colour num max | num == max = color3 0 0 0
               | otherwise  = color3 r g b
                 where
				   ratio = num // max * pi 
				   r = 255 * ( cos (ratio ) )
				   g = 255 * ( cos (ratio  + d) )
				   b = 255 * ( cos (ratio  + 2*d) )
-}
