-- Graphical Julia Set Representation
-- Author: Sebastian Schulze
-- Matric: s1209784

-- This programm produces a visual representation of the Julia set of any function C -> C
-- Furthermore, the user is enabled to move over and zoom in and out of the plane. He can also
-- change the amount of maximum iterations used to determine whether a certain point is part of
-- the specific Julia set or not, thereby increasing the quality of the representation.
-- Currently, every keypress causes the programm to render once again.
-- In a later version this should not be the case in order to improve the performance of the
-- programm. This later version could also let the users choose the color the Julia set is
-- drawn in.

-- To run the programm call mymain in the form:
-- mymain f "[opts]"
			-- f is the function C ->C whose Julia set is to be displayed (e.g. a lamda expression)
			-- opts are the optional parameters (even if you choose to use the default values 
				-- you are required to put the empty string there)

import Graphics.UI.GLUT
import Data.IORef
import Data.Complex
import Bindings -- handles input and imports display functions

-- Parses String of optional parmeters into tupel
-- (maxIterations, zoom, width, height)
-- the default values are set at call in main
parseOptional :: [String] -> (Int, Float, GLsizei, GLsizei) -> (Int, Float, GLsizei, GLsizei)
parseOptional []  c = c
parseOptional [x] (depth, zoom, width, height) = error "Invalid arguments"
parseOptional (x:y:xs) (depth, zoom, width, height)
                                  | x == "-depth"  = parseOptional xs (read y, zoom, width, height)
			   					  | x == "-zoom"   = parseOptional xs (depth,  read y, width, height)
			   					  | x == "-width"   = parseOptional xs (depth, zoom, read y, height)
			   					  | x == "-height"   = parseOptional xs (depth, zoom, width, read y)
								  | otherwise = parseOptional xs (depth, zoom, width, height)

-- Main function - it builts a window and calls the appropriate functions to handle user input and display the specific function
mymain :: (Complex Float -> Complex Float) -> String -> IO() 
mymain f opt = do
  getArgsAndInitialize
  initialDisplayMode $= [SingleBuffered, RGBMode]
  initialWindowSize $= Size width height -- creates a window of the given (or default) size
  initialWindowPosition $= Position 100 100
  createWindow "Julia"
  clearColor $= Color4 0 0 0 0
  matrixMode $= Projection
  loadIdentity
  zoom <- newIORef (initZoom::GLfloat) -- Reference to the current zoom factor
  position <- newIORef (0.0::GLfloat, 0.0) -- Reference to the current shift of the representation
  maxIt <- newIORef (max) -- Reference to the number of maximum iterations (see module julia)
  keyboardMouseCallback $= Just (keyboard position zoom maxIt) -- handels keyboard input
  displayCallback $= (display f maxIt position zoom) -- displays the function with the current parameters
  mainLoop
	where
		optargs = (parseOptional (words opt) (25, 1, 400, 340))
		max = (\(x, _, _, _) -> x) optargs -- first optional parameter containing the maximum numer of iterations while determining the Julia set, default value 25
		initZoom = (\(_, x, _, _) -> x) optargs -- second optional parameter containing the initial zoom factor, default value 1
		width = (\(_, _, x, _) -> x) optargs -- third optional parameter containing the initial window width, default value 400
		height = (\(_, _, _, x) -> x) optargs -- second optional parameter containing the initial window height, default value 340
